# Generated by Django 4.2.1 on 2023-08-28 05:07

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0003_delete_translatable"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="formkitschema",
            name="key",
        ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="translation_context",
        ),
        migrations.AddField(
            model_name="formkitschema",
            name="label",
            field=models.TextField(blank=True, help_text="Used as a human-readable label", null=True),
        ),
        migrations.AlterField(
            model_name="formcomponents",
            name="label",
            field=models.CharField(blank=True, help_text="Used as a human-readable label", max_length=1024, null=True),
        ),
        migrations.AlterField(
            model_name="formcomponents",
            name="node",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="formkit_ninja.formkitschemanode",
            ),
        ),
        migrations.AlterField(
            model_name="formkitschemanode",
            name="label",
            field=models.CharField(blank=True, help_text="Used as a human-readable label", max_length=1024, null=True),
        ),
    ]
