# Generated by Django 4.2.1 on 2023-08-28 05:39

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0007_alter_option_field"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="option",
            name="label",
        ),
    ]
