# Generated by Django 4.2.1 on 2023-08-28 05:26

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0005_option_last_updated_option_object_id_optionlabel_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="option",
            name="group",
            field=models.ForeignKey(
                default="", on_delete=django.db.models.deletion.CASCADE, to="formkit_ninja.optiongroup"
            ),
            preserve_default=False,
        ),
        migrations.AddConstraint(
            model_name="option",
            constraint=models.UniqueConstraint(fields=("group", "object_id"), name="unique_option_id"),
        ),
    ]
