# Generated by Django 4.1.5 on 2023-05-09 05:37

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0002_alter_formcomponents_label_alter_formkitschema_key"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Translatable",
        ),
    ]
